#ifndef __LIBGNQS_NETINT_MESSAGES_HEADER
#define __LIBGNQS_NETINT_MESSAGES_HEADER

/*
 * libgnqs/net-int/messages.h
 *		List of all "messages" which the GNQS process can receive
 *
 *		Part of Generic NQS.
 *
 * Author	Stuart Herbert
 *		(stuart@gnqs.org)
 *
 * Copyright	(c) 1999 Stuart Herbert
 *		Released under v2 of the GNU GPL
 */

/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 * List of all messages for NQS Internal Networking (NQS-INT)
 *
 * HINT:
 *
 *	There is a validation table (in tables.c) for each message type 
 *	supported by the NQS Internal Protocol v1.
 *
 *	The validation table includes a description of what the message
 *	is for.  This can be output to the GNQS log file for debugging
 *	purposes, to aid developers new to the source base.  These
 *	messages appear when:
 *
 *		DEBUG_NQSINT1		is set to 'high'
 *		DEBUG_NQSINT1_MSGDESC	is set to 'on'
 */

/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 * So-called "internal" packet types
 *
 * This is the list of messages which the GNQS daemon will send (and accept)
 * on the well-known UNIX domain socket it listens on.
 *
 * Compatibility from version to version is NOT guaranteed!!
 *
 * In particular, it's quite likely this list will shorten as v2 of the NQS
 * Network Protocol is introduced ...
 */

#define NETINT1_MSG_I_ABOQUE		0
#define NETINT1_MSG_I_ADDFOR		1
#define NETINT1_MSG_I_ADDNQSMAN		2
#define NETINT1_MSG_I_ADDQUEDES		3
#define NETINT1_MSG_I_ADDQUEDEV		4
#define NETINT1_MSG_I_ADDQUEGID		5
#define NETINT1_MSG_I_ADDQUEUID		6
#define NETINT1_MSG_I_CREDEV		7
#define NETINT1_MSG_I_CREBATQUE		8
#define NETINT1_MSG_I_CREDEVQUE		9
#define NETINT1_MSG_I_CREPIPQUE		10
#define NETINT1_MSG_I_DELDEV		11
#define NETINT1_MSG_I_DELFOR		12
#define NETINT1_MSG_I_DELNQSMAN		13
#define NETINT1_MSG_I_DELQUE		14
#define NETINT1_MSG_I_DELQUEDES		15
#define NETINT1_MSG_I_DELQUEDEV		16
#define NETINT1_MSG_I_DELQUEGID		17
#define NETINT1_MSG_I_DELQUEUID		18
#define NETINT1_MSG_I_DELREQ		19
#define NETINT1_MSG_I_DISDEV		20
#define NETINT1_MSG_I_DISQUE		21
#define NETINT1_MSG_I_ENADEV		22
#define NETINT1_MSG_I_ENAQUE		23
#define NETINT1_MSG_I_FAMILY		24
#define NETINT1_MSG_I_LOCDAE		25
#define NETINT1_MSG_I_MODREQ		26
#define NETINT1_MSG_I_MOVREQ		27
#define NETINT1_MSG_I_PURQUE		28
#define NETINT1_MSG_I_QUEREQ		29
#define NETINT1_MSG_I_QUEREQVLPQ	30
#define NETINT1_MSG_I_REQCOM		31
#define NETINT1_MSG_I_RMTACCEPT		32
#define NETINT1_MSG_I_RMTARRIVE		33
#define NETINT1_MSG_I_RMTDEPART		34
#define NETINT1_MSG_I_RMTENADES		35
#define NETINT1_MSG_I_RMTENAMAC		36
#define NETINT1_MSG_I_RMTFAIDES		37
#define NETINT1_MSG_I_RMTFAIMAC		38
#define NETINT1_MSG_I_RMTQUEREQ		39
#define NETINT1_MSG_I_RMTRECEIVED	40
#define NETINT1_MSG_I_RMTSCHDES		41
#define NETINT1_MSG_I_RMTSCHMAC		42
#define NETINT1_MSG_I_RMTSTASIS		43
#define NETINT1_MSG_I_SCHEDULEREQ	44
#define NETINT1_MSG_I_SENSEDAEMON	45
#define NETINT1_MSG_I_SETDEB		46
#define NETINT1_MSG_I_SETDEFBATPR	47
#define NETINT1_MSG_I_SETDEFBATQU	48
#define NETINT1_MSG_I_SETDEFDESTI	49
#define NETINT1_MSG_I_SETDEFDESWA	50
#define NETINT1_MSG_I_SETDEFDEVPR	51
#define NETINT1_MSG_I_SETDEFPRIFO	52
#define NETINT1_MSG_I_SETDEFPRIQU	53
#define NETINT1_MSG_I_SETDEVFOR		54
#define NETINT1_MSG_I_SETDEVSER		55
#define NETINT1_MSG_I_SETFOR		56
#define NETINT1_MSG_I_SETLIFE		57
#define NETINT1_MSG_I_SETLOGFIL		58
#define NETINT1_MSG_I_SETMAXCOP		59
#define NETINT1_MSG_I_SETMAXOPERE	60
#define NETINT1_MSG_I_SETMAXPRISI	61
#define NETINT1_MSG_I_SETNDFBATQU	62
#define NETINT1_MSG_I_SETNDFPRIFO	63
#define NETINT1_MSG_I_SETNDFPRIQU	64
#define NETINT1_MSG_I_SETNETCLI		65
#define NETINT1_MSG_I_SETNETDAE		66
#define NETINT1_MSG_I_SETNETSER		67
#define NETINT1_MSG_I_SETNONETDAE	68
#define NETINT1_MSG_I_UNDEFINED1	69
#define NETINT1_MSG_I_SETNOQUEACC	70
#define NETINT1_MSG_I_SETNQSMAI		71
#define NETINT1_MSG_I_SETNQSMAN		72
#define NETINT1_MSG_I_SETOPEWAI		73
#define NETINT1_MSG_I_SETPIPCLI		74
#define NETINT1_MSG_I_SETPIPONL		75
#define NETINT1_MSG_I_SETPPCORE		76
#define NETINT1_MSG_I_SETPPCPUT		77
#define NETINT1_MSG_I_SETPPDATA		78
#define NETINT1_MSG_I_SETPPMEM		79
#define NETINT1_MSG_I_SETPPNICE		80
#define NETINT1_MSG_I_SETPPPFILE	81
#define NETINT1_MSG_I_SETPPQFILE	82
#define NETINT1_MSG_I_SETPPSTACK	83
#define NETINT1_MSG_I_SETPPTFILE	84
#define NETINT1_MSG_I_SETPPWORK		85
#define NETINT1_MSG_I_SETPRCPUT		86
#define NETINT1_MSG_I_SETPRDRIVES	87
#define NETINT1_MSG_I_SETPRMEM		88
#define NETINT1_MSG_I_SETPRNCPUS	89
#define NETINT1_MSG_I_SETPRPFILE	90
#define NETINT1_MSG_I_SETPRQFILE	91
#define NETINT1_MSG_I_SETPRTFILE	92
#define NETINT1_MSG_I_SETQUEDES		93
#define NETINT1_MSG_I_SETQUEDEV		94
#define NETINT1_MSG_I_SETQUEPRI		95
#define NETINT1_MSG_I_SETQUERUN		96
#define NETINT1_MSG_I_SETSHSFIX		97
#define NETINT1_MSG_I_SETSHSFRE		98
#define NETINT1_MSG_I_SETSHSLOG		99
#define NETINT1_MSG_I_SETUNRQUEAC	100
#define NETINT1_MSG_I_SHUTDOWN		101
#define NETINT1_MSG_I_STAQUE		102
#define NETINT1_MSG_I_STOQUE		103
#define NETINT1_MSG_I_UNLDAE		104
#define NETINT1_MSG_I_SETQUENDP		105
#define NETINT1_MSG_I_SETGBATLIM	106
#define NETINT1_MSG_I_SETQUEUSR		107
#define NETINT1_MSG_I_ADDQUECOM		108
#define NETINT1_MSG_I_CRECOM		109
#define NETINT1_MSG_I_DELCOM		110
#define NETINT1_MSG_I_MOVQUE		111
#define NETINT1_MSG_I_REMQUECOM		112
#define NETINT1_MSG_I_SETCOMLIM		113
#define NETINT1_MSG_I_SETCPUACC		114
#define NETINT1_MSG_I_SUSPENDREQ	115
#define NETINT1_MSG_I_HOLDREQ		116
#define NETINT1_MSG_I_RELREQ		117
#define NETINT1_MSG_I_SETPIPLIM		118
#define NETINT1_MSG_I_VERQUEUSR		119
#define NETINT1_MSG_I_SETQUECHAR	120
#define NETINT1_MSG_I_SETNOQUECHAR	121
#define NETINT1_MSG_I_SETCOMUSERLIM	122
#define NETINT1_MSG_I_MEMDUMP		123
#define NETINT1_MSG_I_LOAD		124
#define NETINT1_MSG_I_RREQCOM		125
#define NETINT1_MSG_I_SETNQSSCHED	126
#define NETINT1_MSG_I_SETDEFLOADINT	127
#define NETINT1_MSG_I_CTLDAE		128
#define NETINT1_MSG_I_SETLOADDAE	129
#define NETINT1_MSG_I_ALTERREQ		130
#define NETINT1_MSG_I_SETSERVPERF	131
#define NETINT1_MSG_I_SETSERVAVAIL	132

/* the following messages have all been added for GNQS v4.0-pre ... */

#define NETINT1_MSG_I_END		133

#endif /* __LIBGNQS_NETINT1_MESSAGES_HEADER */
